;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; GENERAL HOUSING MODEL with BANKS
;;;; J M Applegate
;;;; January 4, 2020
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

globals [
  ;;;; on interface: meanIncome, density, locality, ownershipRate, exitRate, entryRate
  ;;;; interbankRate, requiredLTV, standardMortgageTerm, standardRentalTerm
  ;;;; affordabilityRate, maxAffordability, minConsumptionRate, aspiration, deltaRate, timeMarketThreshold, maxHomeless
  ;;;; time, console
  nBanks
  ownerOccupied
  rentals
  forSale
  forRent
  seekers
  sold
  buyers
  highDemand
  lowDemand
  aveTimeMarket
  medTimeMarket
  aveTimeRentalMarket
  medTimeRentalMarket
  maxIncome
  k
  avePrice
  medPrice
  aveRent
  medRent
  aveSalePrice
  medSalePrice
  aveRentalRent
  medRentalRent
  ;;;; plotting assists
  minPrice
  maxPrice
  minRent
  maxRent
  housesForSale
  housesSold
  housesForRent
  housesRented
]

breed [ households household ]
breed [ houses house ]
breed [ banks bank ]
breed [ sale-records sale-record ]
breed [ bank-records bank-record ]
breed [ rental-records rental-record ]

households-own [
  residence
  income
  lender
  capital
  madeOffer
  affordableRent
  affordablePrice
  timeHomeless
  consumption
  affordability
  accustomed
  ]

houses-own [
  owner
  inhabitant
  quality
  status
  askingPrice
  askingRent
  price
  rent
  buyer
  seller
  rentalTerm
  payment
  mortgage
  dateMarket
  offerFrom
  saleDate
  rentalDate
  ]

banks-own [
  minLTV
  term
  interestRate
  interestMultiplier
  ledger
  profit
  ]

sale-records-own [
  property
  buyer
  seller
  salePrice
  saleDate
  timeOnMarket
  ]

bank-records-own [
  property
  client
  date
  initialMortgage
  loanInterest
  loanTerm
  mortgageBalance
  mortgagePayment
  numberPayments
  loanStatus
  ]

rental-records-own [
  property
  rentalPrice
  rentalTerm
  rentalDate
  timeOnMarket
  ]

to setup

  clear-all
  reset-ticks

  set-default-shape houses "house"

 ; no-display

  ask patches [ set pcolor 98 ] ;;; 98 is pale blu

  build-banks

  set aveTimeMarket timeMarketThreshold
  set medTimeMarket timeMarketThreshold
  set aveTimeRentalMarket timeMarketThreshold
  set medTimeRentalMarket timeMarketThreshold

  ;;;; construct houses according to density parameter
  repeat ( count patches * density / 100 ) [ build-house ]

  ;;;; generate household for all houses and assign income, use this income information to set
  ;;;; initial price, quality, mortgage, payment and rent values for each house.

  generate-households

  set avePrice mean [ price ] of houses
  set medPrice median [ price ] of houses

  ;;;; assign each house a quality based on price
  ask houses [
     set quality price / medPrice
     if quality > 3 [set quality 3] if quality < 0.3 [set quality 0.3]
     ]

  ;;;; if wanting to use spatial dimension, need to groups houses of similar price . . .
  ;;;; Gilbert provides cluster algorithm for spatially clustering houses with similar income
  ;;;; requires sufficient empty space to be effective, seek alternative . . .
  cluster

  ;;;; set owner occupied residences and inhabitants
  set ownerOccupied n-of (ownershipRate / 100 * count houses) houses
  ask ownerOccupied [
    set color 95 ; 95 is royal blue
    set inhabitant owner
    ask owner [ set residence myself ]
    set status "owner occupied"
  ]

  ;;;; assign rental houses owners from current homeowners and
  set rentals houses with [ not member? self ownerOccupied ]
  ask rentals [
    set color 55 ; 55 is green
    set inhabitant owner
    set owner one-of households with [ is-house? residence and income > meanIncome / 6 ] ; greater than twice mean income . . . .
    ask inhabitant [ set residence myself ]
    set rentalTerm random standardRentalTerm
    set status "tenant occupied"
    hatch-rental-records 1 [
      hide-turtle
      set property myself
      set rentalPrice [ rent ] of myself
      set rentalTerm [ rentalTerm ] of myself
      set rentalDate 0
      set timeOnMarket medTimeRentalMarket
    ]
  ]

  ask households [ set accustomed [ quality ] of residence ]

  set highDemand 1
  set lowDemand  1 - deltaRate / 100

  if any? ownerOccupied [
    set avePrice mean [ price ] of ownerOccupied
    set medPrice median [ price ] of ownerOccupied
  ]
  if any? rentals [
    set aveRent mean [ rent ] of rentals
    set medRent median [ rent ] of rentals
  ]

  ;;;; Gilbert provides algorithm to color house patches on a gradient according to price
  paint-houses

  if console [
    ask households [
      type "household " type who type " has capital " type round capital type " and income " print round income
      type "   with mortgage " type [ mortgage ] of residence type ", mortgage payment " type round ( [ payment ] of residence ) type " and rent " print round ( [ rent ] of residence )
    ]
  ]

end

to build-banks
  create-banks numberBanks [
    hide-turtle
    set interestRate interbankRate / 100 + .02
    set minLTV requiredLTV / 100
    set term standardMortgageTerm
    let J interestRate
    let n term * 12
    set interestMultiplier J * (1 + J) ^ n / ((1 + J) ^ n - 1)
    set ledger []
  ]
end

to build-house
  create-houses 1 [
    set color gray
    move-to one-of patches
    if count houses-here > 1 [
      let emptySites patches with [ not any? houses-here ]
      if any? emptySites [ move-to one-of emptySites ]
      ]
  ]
end

to generate-households
  ask houses [
    hatch-households 1 [
      ask myself [ set owner myself ]
      assign-income
      set lender one-of banks
    ]
    set payment [ income ] of owner * [ affordability ] of owner
    set rent payment * ( 1 + random ( aspiration / 100 ) )
    let principal payment / [ [ interestMultiplier ] of lender ] of owner
    set mortgage random principal
    set price principal  / [ [ minLTV ] of lender ] of owner
    hatch-sale-records 1 [
        hide-turtle
        set property myself
        set buyer [ owner ] of myself
        set seller [ owner ] of myself
        set salePrice [ price ] of myself
        set saleDate -1 * random 60
        set timeOnMarket medTimeMarket
    ]
    ifelse mortgage < payment
    [ set payment 0
      set mortgage 0 ]
    [ ask [ lender ] of owner [ create-bank-record  myself ]]
  ]
end

to assign-income
;;;; from Gilbert -
;;;; assigns the income and savings of a new household
;;;; uses gamma (alpha, lambda) distribution
;;;; mean of a gamma distribution = alpha / lambda
;;;; parameters taken from http://www2.physics.umd.edu/~yakovenk/papers/PhysicaA-370-54-2006.pdf
  let alpha 1.3
  let lambda alpha / meanIncome
  let annualIncome 0
  ;avoid impossibly low incomes (i.e. less than a third the desired mean income) ???
  while [ annualIncome < meanIncome / 3 ] [
  set annualIncome ( meanIncome * lambda / alpha ) * ( random-gamma alpha lambda )
    ]
  set capital random 100 / 100 * annualIncome
  set income annualIncome / 12
  set affordability affordabilityRate / 100
  set-consumption
end

to set-consumption
  ask households [
    ;set consumption ( 1 - affordability - minConsumptionRate / 100 ) / ( 1 + exp ( k * ( income * 12 - meanIncome ) ) ) + minConsumptionRate / 100
    let rate min list 1 ( -.2 * ( income ) ^ .2 + 2 ) ;;; parametrized to US 2012 pre-tax CEX data
    set consumption rate - affordability
  ]
end

to cluster
  repeat 5 [
    let housesToMove sort-by [ [?1 ?2] -> priceDiff ?1 > priceDiff ?2 ] houses
    foreach housesToMove [ ?1 ->
      if priceDiff ?1 > 10000 [
        let vacantPlot one-of patches with [ not any? houses-here and
                                  abs (localPrice - [price] of ?1) < 10000 ]
        if vacantPlot != nobody [
          ask ?1 [ move-to vacantPlot ]
          ]
        ]
      ]
    ]
    ask houses [
      if is-household? owner [ ask owner [ move-to myself ] ]
      ]
end

to-report priceDiff [ a-house ]
  report abs ([price] of a-house - [localPrice] of a-house)
end

to-report localPrice
  let localHouses houses with [ distance myself < locality ]
  ifelse any? localHouses
    [ report median [ price ] of localHouses ]
    [ report 0 ]
end

to paint-houses       ;; observer procedure
;; recolor patches to show the current sale price of houses
; use log sale -price to get a better variation in colour
  let lnMinPrice ln [ price ] of min-one-of houses [ price ]
  let lnMaxPrice ln [ price ] of max-one-of houses [ price ]
  ask houses [
    ;set pcolor palette:scale-scheme "Divergent" "RdYlBu" 10 (ln price) lnMinPrice lnMaxPrice
    set pcolor scale-color green ( ln price ) lnMinPrice lnMaxPrice
    ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Here the model runs . . .
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to go
  step
  do-plots
  tick
  if ticks = time [ stop ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; action body of code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to step

  if console [ type "STEP " print ticks ]

;;;; households collect wage and rental incomes and pay mortgages and rents

  collect-income-pay-costs

;;;; update bank payment and mortgage balance information

  update-bank-ledgers

  if console [
    print "*** selling market ***"
  ]

;;;; determine houses for sale this time step, new listing and unsold from last time step
; *************************************************************************
; for investment, how to add rental houses to list of forSale?
; *************************************************************************

  set forSale ( turtle-set n-of ( exitRate / 100 * count ownerOccupied ) ownerOccupied houses with [ status = "for sale" ] )

  ;;;; base median sale price and time on market on last year
  if any? sale-records
    [ let currentRecords sale-records with [ saleDate > ticks - 12 ]
      set aveTimeMarket mean [ timeOnMarket ] of currentRecords
      set medTimeMarket median [ timeOnMarket ] of currentRecords
      set aveSalePrice mean [ salePrice ] of currentRecords
      set medSalePrice median [ salePrice ] of currentRecords ]

  if console [
    type "average price of houses is " type round avePrice type " and median price is " print round medPrice
    type "average time on market is " type round aveTimeMarket type " and median is " print round medTimeMarket
    if any? sale-records [ type "average sale price of houses is " type round aveSalePrice type " and median sale price is " print round medSalePrice ]
  ]

  ;;;; Here we're using locality as the determinant of house price.
  ;;;; For a non-spatial model one could use sales or houses of similar quality.
  if any? forSale [
    ask forSale [
      if status = "owner occupied" [
        set color 15 ; 15 is red
        set status "for sale"
        set dateMarket ticks
        let localHouses ownerOccupied with [ distance myself < locality ]
        let currentRecords sale-records with [ saleDate > ticks - 12 ]
        let localSales localHouses with [ member? self turtle-set [ property ] of currentRecords ]
        ifelse any? localHouses ;;;; house prices are sticky downward, owners will attempt to get perceived value even though market price is lower
          [ ifelse any? localSales
              ;[ set askingPrice max list price mean [ price ] of localSales ]
              [ set askingPrice max list price median [ price ] of localSales ]
              ;[ set askingPrice max list price mean [ price ] of localHouses ] ]
              [ set askingPrice max list price median [ price ] of localHouses ] ]
          [ set askingPrice max list price aveSalePrice ]
        set askingPrice askingPrice * ( 1 + random ( aspiration / 100 ) )
      ]
    ]
  ]

  ;;;; set market-drive price adjustments
  ifelse medTimeMarket <= timeMarketThreshold
  ;;;; compounds the high demand increases
    [ set highDemand highDemand * ( 1 + deltaRate / 100 )
      if console [ type "demand is high, prices raised by " print precision highDemand 3 ] ]
    [ set highDemand 1 ]

  ;;;; adjust prices
  if any? forSale [
    ask forSale [
      let currentTime ticks - dateMarket
      if currentTime = 0 [
        set askingPrice askingPrice * highDemand ]
      if ( currentTime + 1 ) mod timeMarketThreshold = 0 [
          set askingPrice askingPrice * lowDemand
          if console [ type "    house " type who type " reduced asking price to " print round askingPrice ]
        ]
      ]
  ]

  if console [
    print "houses for sale:"
    if any? forSale [ ask forSale [ type "   house " type who type " with owner " type owner type ", value " type round price type " and asking " print round askingPrice ] ]
  ]

  ; ********************************************************************************************************
  ; ********* Investment market
  ; ********* determine investore
  ; ********* determine investor options
  ; ********* investors make offers
  ; ********* offers accepted or rejected
  ; ********* transfer ownership of investment property, set payment and mortgage, generate sales-record
  ; ********* set house status "for rent" and rentalTerm = 0
  ; ********************************************************************************************************

  ;;;; sellers market
  ;;;; assume preference for buying, household will rent only if they cannot buy
  ;;;; in future could make rent or buy choice based on highest affordable quality; either sell house or use as income property
  ;;;; choose house with highest quality if multiple options
  ;;;; single offer made each timestep if there are options
  ;;;; maybe a quality threshold based on current residence . . .

  ;;;; seeking households determine affordable sale and rents
  set seekers households with [ not is-house? residence or [ status ] of residence = "for sale" ]
  ask seekers [
    set affordablePrice ( income * affordability ) / ( [ interestMultiplier ] of lender * [ minLTV ] of lender )
  ]

  if console [
    print "seekers:"
    if any? seekers [ ask seekers [ type "   household " type who type " with affordable price " type  round affordablePrice type " and funds " print round ( [ capital ] of self + equity [ residence ] of self ) ] ]
  ]

  ask seekers [
    let options forSale with [
      not is-household? offerFrom and
      owner != myself and
      askingPrice <= [ affordablePrice ] of myself and
      [ capital ] of myself + equity [ residence ] of myself >= deposit myself self
    ]
    if any? options [
      if console [ type "household " type who type " has options: " print [ self ] of options ]
      let choice one-of options with [ quality = max [ quality ] of options ]
      set madeOffer choice
      ask choice [ set offerFrom myself ]
    ]
  ]

  ;;;; Generate list of houses with offers sorted from low to high mortgage balances, so higher equity buyers likely at front of list.
  ;;;; For each item in list, if offer is accepted and sale can complete, complete sale and remove house from list.
  ;;;; If offer is accepted but sale cannot complete move house to end of list.
  ;;;; Once list no longer shrinks, all possible transactions have occurred
  let withOffers sort-on [ mortgage ] houses with [ is-household? offerFrom ]

  if console [
    ifelse empty? withOffers
    [ print "no buying offers this step" ]
    [ foreach withOffers [ a-house -> ask a-house [ type "house " type who type " received offer from " print offerFrom ] ] ]
  ]

  set sold no-turtles
  set buyers no-turtles

  ;;;; sell houses

  if not empty? withOffers [
    let startList 0
    let endList 1
    while [ startList != endList ] [
      set startList withOffers
      foreach withOffers
      [ a-house -> ask a-house [
        set withOffers remove a-house withOffers
        ;;;; if offer accepted
        ifelse [ affordablePrice ] of offerFrom >= askingPrice
          ;;;; if contract is executable
          [ ifelse [ capital ] of offerFrom >= deposit offerFrom a-house
            [ if console [ type "house " type who type " sells from " type owner type " to " print offerFrom ]
              set sold ( turtle-set sold self )
              set buyers ( turtle-set buyers [ offerFrom ] of self )
              sell-house ]
            [ if console [ type "house " type who type " in chain " type owner type " with " print offerFrom ]
              set withOffers lput a-house withOffers ] ]
          [ if console [ type "house " type who type " with owner " type owner type " rejected offer from " print offerFrom ]
            ask offerFrom [ set madeOffer 0 ]
            set offerFrom 0 ]
        ]
      ]
        set endList withOffers
      ]
    ]

  ask sold [ move ]

  if console [
    print "unsold houses this step:"
    ask houses with [ status = "for sale" ] [ type "   house " print who ]
    print "*** rental market  ***"
  ]

  ;;;; rent houses
  set forRent rentals with [ rentalTerm = 0 or status = "for rent" ]

  if any? rental-records
    [ let currentRecords rental-records with [ rentalDate > ticks - 12 ]
      set aveTimeRentalMarket mean [ timeOnMarket ] of currentRecords
      set medTimeRentalMarket median [ timeOnMarket ] of currentRecords
      set aveRentalRent mean [ rentalPrice ] of currentRecords
      set medRentalRent median [ rentalPrice ] of currentRecords ]

  if console [
    type "average rent of houses is " type round aveRent type " and median rent is " print round medRent
    type "average time on market is " type round aveTimeRentalMarket type " and median is " print round medTimeRentalMarket
    if any? rental-records [ type "average rental rent of houses is " type round aveRentalRent type " and median rental rent is " print round medRentalRent ]
  ]

  if any? forRent [
    ask forRent [
      if status = "tenant occupied" [
        set color 135 ; 135 is pink
        set status "for rent"
        set dateMarket ticks
        ask inhabitant [ set residence nobody ]
        set inhabitant nobody
        let localHouses rentals with [ distance myself < locality ]
        let currentRecords rental-records with [ rentalDate > ticks - 12 ]
        let localRentals localHouses with [ member? self turtle-set [ property ] of currentRecords ]
        ifelse any? localHouses
          [ ifelse any? localRentals
              [ set askingRent mean [ rent ] of localRentals ]
              [ set askingRent mean [ rent ] of localHouses ] ]
          [ set askingRent aveRentalRent ]
        set askingRent askingRent * ( 1 + random ( aspiration / 100 ) )
      ]
    ]
  ]

  ifelse medTimeRentalMarket <= timeMarketThreshold
  [ set highDemand ( 1 + deltaRate / 100 )
    if console [ type "demand is high, rents raised by " print precision highDemand 3 ] ]
  [ set highDemand 1 ]

  ;;;; adjust rental prices
  if any? forRent [
    ask forRent [
      let currentTime ticks - dateMarket
      if currentTime = 0 [ set askingRent askingRent * highDemand ]
      if ( currentTime + 1 ) mod timeMarketThreshold = 0 [
        set askingRent askingRent * lowDemand
        if console [ type "    house " type who type " reduced asking rent to " print round askingRent ]
      ]
    ]
  ]

  if console [
    print "houses for rent:"
    if any? forRent [ ask forRent [ type "   house " type who type " with rental term " type rentalTerm type " and asking rent " print round askingRent ] ]
  ]

  ;;;; anyone without a residence seeks rental
  let renters households with [ not is-house? residence ]

  ask renters [ set affordableRent income * affordability ]

  if console [
    print "renters:"
    if any? renters [ ask renters [ type "   household " type who type " with affordable rent " print round affordableRent ] ]
  ]

  ask renters [
    let options forRent with [
      not is-household? offerFrom and
      askingRent <= [ affordableRent ] of myself
    ]
    ;set options up-to-n-of searchLength options
    if any? options [
      if console [ type "renter " type who type " has options: " print [ self ] of options ]
      let choice one-of options with [ quality = max [ quality ] of options ]
      set madeOffer choice
      ask choice [ set offerFrom myself ]
    ]
  ]

  set withOffers no-turtles
  set withOffers houses with [ is-household? offerFrom ]

  if console [
    ifelse any? withOffers
    [ ask withOffers [ type "house " type who type " received offer from " print offerFrom ] ]
    [ print "no rental offers this step" ]
  ]

  if any? withOffers [
    ask withOffers [
      ;;;; if offer accepted
      ifelse [ affordableRent ] of offerFrom >= askingRent
        [ if console [ type "house " type who type " is rented by " print offerFrom ]
          rent-house ]
      [ if console [ type "house " type who type " rejected offer from " print offerFrom ]
        ask offerFrom [ set madeOffer 0 ]
        set offerFrom 0 ]
      ]
    ]

  ;;;; household consumption
  ;; set maxIncome max [ income ] of households
  ;; set k 10 ^ ( -1 * floor ( ( log ( maxIncome * 12 ) 10 ) + 1 ) )
  ask households [
    set-consumption
    set capital capital - consumption * income
    ;if console [ type "affordability is " type affordability type " and consumption is " type consumption type " for total consumption " show affordability + consumption ]
  ]

  ;;;; update counters and values
  ask rentals [ set rentalTerm rentalTerm - 1 ]
  ask households with [ not is-house? residence ] [ set timeHomeless timeHomeless + 1 ]

  if console [
    print "******"
    print "vacant houses this step:"
    ask houses with [ not is-household? inhabitant ] [ type "   house " print who ]
    print "homeless households this step:"
    ask households with [ not is-house? residence ] [ type "   household " print who ]
  ]

  ;;;; update affordability if homeless past threshhold
  ask households [
    if timeHomeless > 0 and timeHomeless mod maxHomeless = 0 and affordability < 1 - minConsumptionRate / 100 [
      if console [ type "household " type who type " increased its housing affordabilty from " type precision affordability 3 ]
      set affordability affordability * ( 1 + deltaRate / 100 )
      if console [ type " to " type precision affordability 3 type " at time homeless " print timeHomeless ]
    ]
  ]

  if console [ print "" ]

  ;;;; TODO update demographics; some households leave, some arrive
  ;;;; TODO add investment decisions; some owner occupied become rentals and some rentals become owner occupied
  ;;;; TODO deconstruct and construct houses according to quality demand


; *********************************************************************************************************
; for investment need to think through what is rental and what is owner occupied in finding median prices
; *********************************************************************************************************

;  let landlords no-turtles
;  let homeowners no-turtles
;  ask rentals [ set landlords ( turtle-set landlords owner ) ]
;  ask ownerOccupied [ set homeowners ( turtle-set homeowners owner ) ]
;  set homeowners homeowners with [ not member? self landlords ]
  ;https://stackoverflow.com/questions/40078184/having-turtle-specific-variables-from-two-different-breeds-of-the-same-name

  ;let landlords turtle-set [ owner ] of rentals
  ;show count landlords

end

to-report equity [ a-house ]
  ifelse is-house? a-house
  [ report [ price ] of a-house  - [ mortgage ] of a-house ]
  [ report 0 ]
end

to-report deposit [ a-household a-house ]
  report ( 1 - [ [ minLTV ] of lender ] of a-household ) * [ askingPrice ] of a-house
end

;;; set up reporters for payments and mortgages

to collect-income-pay-costs
  ask households [
    set capital capital + income
    let properties houses with [ owner = myself ]
    ;if console [ type "household " type who type " with capital " type capital type " and income " type income type " owns " type [ self ] of properties type " with total mortgage payments of " print sum [ payment ] of properties ]
    if any? properties [
      let rentalIncome sum [ rent ] of properties with [ status = "tenant occupied" ]
      set capital capital + rentalIncome
      set capital capital - sum [ payment ] of properties
      ask properties [ set mortgage mortgage - payment ]
    ]
    if not any? properties and is-house? residence [ set capital capital - [ rent ] of residence ]
  ]
end

to update-bank-ledgers  ;observer procedure
  ask banks [
    foreach ledger [
      [ a-record ] -> ask a-record [
        if mortgagePayment != 0 [
          ifelse mortgageBalance > mortgagePayment
            [ set mortgageBalance mortgageBalance - mortgagePayment
              set numberPayments numberPayments - 1 ]
            [ set mortgageBalance 0
              set mortgagePayment 0
              set numberPayments 0
              ask property [
                set mortgage 0
                set payment 0 ]
          ]
        ]
      ]
    ]
  ]
end

to sell-house ;;; house procedure
  set seller owner
  set buyer offerFrom
  ask buyer [ set madeOffer 0 ]
  set offerFrom 0
  set price askingPrice
  ask seller [ set capital capital + equity myself ]
  ask buyer [ get-mortgage  myself ]
  set saleDate ticks + 1
  set owner buyer
  set status "owner occupied"
  if console [ type "house " type who type " sells for " type round price type " with monthly payment " type round payment type " and mortgage "  print round mortgage ]
  set color 95 ; 95 is blue
  create-sale-record
end

to get-mortgage [ a-house ];; buyer procedure
  ifelse [ price ] of a-house < capital / 2
    [ set capital capital - [ price ] of a-house
      ask a-house
        [ set mortgage 0
          set payment 0 ] ]
    [ let downpayment max list ( deposit self a-house ) ( capital / 2 )
      let principal [ price ] of a-house - downpayment
      ask a-house
        [ set mortgage principal
          set payment principal * [ [ interestMultiplier ] of lender ] of myself ]
      ask lender [ create-bank-record  a-house ] ]
end

to move
  ;;;; sort out residences and inhabitants
  ;;;; if seller moves out of own house and ish't a buyer of another, seller is homeless
  if [ residence ] of seller = self and not member? seller buyers [ ask seller [ set residence nobody ] ]
  ;;;; if buyer moves out of house and that house isn't sold, the house's inhabitant is nobody
  ;;;; buyer moves into house
  if is-house? [ residence ] of buyer and not member? [ residence ] of buyer sold [ ask [ residence ] of buyer [ set inhabitant nobody ] ]
  ask buyer [
    set residence myself
    set accustomed [ quality ] of myself
    set timeHomeless 0
  ]
  set inhabitant buyer
  set buyer nobody
  set seller nobody
end

to rent-house ;; house procedure
  ask offerFrom [
    set madeOffer 0
    set residence myself
    set timeHomeless 0
  ]
  set inhabitant offerFrom
  set offerFrom 0
  set status "tenant occupied"
  set rent askingRent
  ;set askingRent 0
  set rentalTerm standardRentalTerm
  create-rental-record
  set color 55 ; 55 is green
end

to create-sale-record ;; house procedure
  hatch-sale-records 1 [
    hide-turtle
    set property myself
    set buyer [ buyer ] of myself
    set seller [ seller ] of myself
    set salePrice [ price ] of myself
    set saleDate ticks
    set timeOnMarket saleDate - [ dateMarket ] of myself + 1
  ]
end

to create-bank-record [ a-house ] ;; bank procedure
  hatch-bank-records 1 [
    hide-turtle
    set property a-house
    set client [ buyer ] of a-house
    set date ticks
    set initialMortgage [ mortgage ] of a-house
    set mortgagePayment [ payment ] of a-house
    set mortgageBalance initialMortgage
    set loanTerm [ term ] of myself
    set loanInterest [ interestRate ] of myself
    set loanStatus "current"
    set numberPayments loanTerm
    ask myself [ add-bank-record myself ]
  ]
end

to create-rental-record ;; house procedure
  hatch-rental-records 1 [
    hide-turtle
    set property myself
    set rentalPrice [ rent ] of myself
    set rentalTerm [ rentalTerm ] of myself
    set rentalDate ticks
    set timeOnMarket rentalDate - [ dateMarket ] of myself + 1
  ]
end

to add-bank-record [ the-record ]
  set ledger fput the-record ledger
end

to do-plots

  ;;;; Income distribution plot
  if any? households [
   set-current-plot "Income distribution"
   set-plot-pen-interval 1000
   set-plot-x-range 0 precision ( max [ income ] of households * 12 ) -5
   histogram [ income * 12 ] of households
  ]

;  ;;;; Capital distribution plot
;  if any? households [
;   set-current-plot "Capital distribution"
;   set-plot-pen-interval 1000
;   set-plot-x-range 0 precision ( max [ capital ] of households ) -5
;   histogram [ capital ] of households
;  ]

  ;;;; Helpers

  if any? ownerOccupied [
    set minPrice precision ( min [ price ] of ownerOccupied ) 0
    set maxPrice precision ( max [ price ] of ownerOccupied ) 0
  ]
  set housesSold no-turtles
  if any? sale-records [ set housesSold sale-records with [ saleDate > ticks - 12 ] ]

  if any? rentals [
    set minRent precision ( min [ rent ] of rentals ) 0
    set maxRent precision ( max [ rent ] of rentals ) 0
  ]
  set housesRented no-turtles
  if any? rental-records [ set housesRented rental-records with [ rentalDate > ticks - 12 ] ]

;  ;;;; House price distribution plot
;  if any? housesForSale [
;  set-current-plot "House price distribution"
;  set-plot-pen-interval 1000
;  set-current-plot-pen "All"
;  histogram [ price ] of ownerOccupied
;  set-current-plot-pen "For sale"
;  histogram [ askingPrice ] of housesForSale
;  set-current-plot-pen "Sold"
;  histogram [ salePrice ] of housesSold
;  ]

  ;;;; House prices over time plot
  set-current-plot "House Prices"
  ;if any? ownerOccupied [ set-plot-y-range precision ( min [ price ] of ownerOccupied ) 0 precision ( max [ price ] of ownerOccupied ) 0 ]
  set-current-plot-pen "All"
  ifelse any? ownerOccupied
    [ plot median [ price ] of ownerOccupied ]
    [ plot 0 ]
  set-current-plot-pen "For Sale"
  ifelse any? forSale
    [ plot median [ askingPrice ] of forSale ]
    [ plot 0 ]
  set-current-plot-pen "Sold"
  ifelse any? housesSold
    [ plot median [ salePrice ] of housesSold ]
    [ plot 0 ]

  ;;;; Rents over time plot
  set-current-plot "Rents"
  ;if any? rentals [ set-plot-y-range minRent maxRent ]
  set-current-plot-pen "All"
  ifelse any? rentals
    [ plot median [ rent ] of rentals ]
    [ plot 0 ]
  set-current-plot-pen "For Rent"
  ifelse any? forRent
    [ plot median [ rent ] of forRent ]
    [ plot medRent ]
  set-current-plot-pen "Rented"
  ifelse any? housesRented
    [ plot median [ rentalPrice ] of housesRented ]
    [ plot 0 ]

  ;;;; Homeless, vacant owner occupied houses and vacant rentals over time
  if ticks mod 1 = 0 [
    set-current-plot "Vacancies and Homeless"
    set-current-plot-pen "Homeless"
    plotxy ticks count households with [ not is-house? residence ]
    set-current-plot-pen "Empty Houses"
    plotxy ticks count ownerOccupied with [ not is-household? inhabitant ]
    set-current-plot-pen "Empty Rentals"
    plotxy ticks count rentals with [ not is-household? inhabitant ]
  ]

;  ;;;; Affordability ratio by income
;  set-current-plot "Affordability Ratio"
;  set-current-plot-pen "Affordability Ratio"
;  plot-pen-reset
;  foreach sort households with [ is-house? residence ] [
;    [ a-household ] -> ask a-household [
;      plotxy [ income ] of self [ affordabilityRatio ] of self
;    ]
;  ]

  ;;;; Affordability ratio by income
  set-current-plot "Affordability Ratio"
  plot-pen-reset
  ask households with [ is-house? residence and ( [ payment ] of residence > 0 or [ rent ] of residence > 0 ) ] [
    if member? residence ownerOccupied [ set-current-plot-pen "Houses" ]
    if member? residence rentals [ set-current-plot-pen "Rentals" ]
     plotxy [ income  ] of self [ affordabilityRatio ] of self
  ]

  ;;;; Bank profit over time
  set-current-plot "Bank Profit"
  ask banks [
  create-temporary-plot-pen ( word who )
  set-plot-pen-color color
  plot bankProfit
  ]

  ;;;; Time on market over time
  set-current-plot "Median Time on Market"
  set-current-plot-pen "Sales"
  plot medTimeMarket
  set-current-plot-pen "Rental"
  plot medTimeRentalMarket

  ;;;; Affordability and consumption rates
  set-current-plot "Total Consumption"
  plot-pen-reset
  ask households [
  set-current-plot-pen "Affordability + Consumption"
  let total [ consumption + affordability ] of self
  plotxy [ income ] of self min list 1 total
  ]

end

to-report affordabilityRatio ;;
  if member? residence ownerOccupied [ report [ payment ] of residence / income ]
  if member? residence rentals [ report [ rent ] of residence / income ]
end

to-report bankProfit ;; sale-record procedure
  let total 0
  foreach ledger [
    [ a-record ] -> ask a-record [
      if mortgagePayment != 0 [
        set total total + mortgagePayment * loanInterest
      ]
    ]
  ]
  report total
end
@#$#@#$#@
GRAPHICS-WINDOW
257
14
748
506
-1
-1
23.0
1
10
1
1
1
0
0
0
1
-10
10
-10
10
1
1
1
ticks
30.0

SLIDER
19
198
234
231
exitRate
exitRate
0
50
10.0
1
1
%
HORIZONTAL

SLIDER
19
56
234
89
meanIncome
meanIncome
0
100000
40000.0
1000
1
pa
HORIZONTAL

TEXTBOX
18
8
219
48
General Housing Model
16
104.0
1

BUTTON
370
515
461
548
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
370
593
461
626
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
19
234
234
267
entryRate
entryRate
0
50
0.0
1
1
%
HORIZONTAL

PLOT
764
14
1067
232
House Prices
time
price
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"All" 1.0 0 -16777216 true "" ""
"For Sale" 1.0 0 -2674135 true "" ""
"Sold" 1.0 0 -13791810 true "" ""

SLIDER
18
613
233
646
deltaRate
deltaRate
0
10
5.0
1
1
%
HORIZONTAL

SLIDER
19
541
234
574
maxHomeless
maxHomeless
0
10
12.0
1
1
ticks
HORIZONTAL

SLIDER
18
686
233
719
maxHomeless
maxHomeless
0
12
12.0
1
1
mos
HORIZONTAL

TEXTBOX
20
37
211
55
Macroeconomic Parameters
14
0.0
1

SLIDER
20
332
235
365
requiredLTV
requiredLTV
0
100
80.0
1
1
%
HORIZONTAL

PLOT
489
513
747
657
Income distribution
annual income
number
0.0
1000000.0
0.0
10.0
true
false
"" ""
PENS
"default" 1000.0 1 -16777216 true "" ""

SLIDER
20
368
235
401
standardMortgageTerm
standardMortgageTerm
0
30
20.0
1
1
yrs
HORIZONTAL

BUTTON
370
554
461
587
One Tick
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
20
296
235
329
interbankRate
interbankRate
0
50
1.0
1
1
%
HORIZONTAL

SLIDER
19
91
234
124
density
density
0
100
75.0
1
1
%
HORIZONTAL

SLIDER
19
162
234
195
ownershipRate
ownershipRate
0
100
65.0
1
1
%
HORIZONTAL

SLIDER
19
126
234
159
locality
locality
0
50
3.0
1
1
NIL
HORIZONTAL

SLIDER
20
403
235
436
standardRentalTerm
standardRentalTerm
1
60
12.0
1
1
mo
HORIZONTAL

SLIDER
18
577
233
610
aspiration
aspiration
0
50
10.0
1
1
%
HORIZONTAL

SWITCH
256
607
346
640
console
console
1
1
-1000

SLIDER
20
466
235
499
affordabilityRate
affordabilityRate
0
100
33.0
1
1
%
HORIZONTAL

SLIDER
19
539
234
572
minConsumptionRate
minConsumptionRate
0
50
30.0
1
1
%
HORIZONTAL

SLIDER
18
649
233
682
timeMarketThreshold
timeMarketThreshold
0
12
3.0
1
1
mos
HORIZONTAL

INPUTBOX
257
516
346
576
time
1200.0
1
0
Number

TEXTBOX
260
581
309
599
in months
10
0.0
1

PLOT
764
231
1067
448
Rents
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"All" 1.0 0 -16777216 true "" ""
"For Rent" 1.0 0 -2064490 true "" ""
"Rented" 1.0 0 -10899396 true "" ""

TEXTBOX
22
447
223
465
Behavioural Parameters
14
0.0
1

TEXTBOX
21
277
208
295
Policy Parameters
14
0.0
1

SLIDER
20
502
235
535
maxAffordability
maxAffordability
0
100
80.0
1
1
%
HORIZONTAL

PLOT
1067
14
1370
231
Affordability Ratio
income
affordability ratio
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"Houses" 1.0 2 -13791810 true "" ""
"Rentals" 1.0 2 -955883 true "" ""

PLOT
1067
231
1370
448
Vacancies and Homeless
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Homeless" 1.0 0 -16777216 true "" ""
"Empty Houses" 1.0 0 -2674135 true "" ""
"Empty Rentals" 1.0 0 -2064490 true "" ""

PLOT
1369
14
1672
231
Bank Profit
time
profit
0.0
10.0
0.0
10.0
true
true
"" ""
PENS

MONITOR
488
663
565
708
households
count households
0
1
11

MONITOR
669
663
746
708
rentals
count rentals
0
1
11

MONITOR
569
663
665
708
owner occupied
count ownerOccupied
0
1
11

SLIDER
253
686
371
719
numberBanks
numberBanks
0
20
1.0
1
1
NIL
HORIZONTAL

PLOT
763
448
1067
665
Median Time on Market
time
length of time
0.0
10.0
1.0
10.0
true
true
"" ""
PENS
"Sales" 1.0 0 -2674135 true "" ""
"Rental" 1.0 0 -2064490 true "" ""

BUTTON
370
633
461
666
Reset Plots
clear-all-plots
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
1067
448
1370
665
Total Consumption
income
consumption
0.0
10.0
0.5
1.0
true
false
"" ""
PENS
"Affordability + Consumption" 1.0 2 -5298144 true "" ""

@#$#@#$#@
## WHAT IS IT?

The General Housing Model demonstrates a basic housing market with bank lending, renters, owners and landlords. This model was developed as a base to which students contributed additional functions during Arizona State University's 2020 Winter School: Agent-Based Modeling of Social-Ecological Systems.


## EXTENDING THE MODEL

This version of the General Housing Model is intended for instruction, and here we outline several potential submodels, each of which could be written or integrated within a week-long course. These outlines are suggestions, as there are many possible algorithms that could address these functions.
#### Population Evolution
The model population could evolve in several dimensions. The households themselves could age, acquiring higher incomes over time and eventually die off, with young households being instantiated. Thus the population could expand or collapse. Population evolution could also be modeled as households leaving and entering at specified rates. The current model will have household savings grow to the point where mortgages are no longer requires, so alternative capital accumulation and dissipation processes could be explored. Income distributions and demographic changes can all be modeled using empirical data for specific cities or regions.
#### Investment Market
Houses with capital above a certain threshold could consider buying a house that would then be offered for rent. The investment purchasing decision would be based on expected rental income, and could be as simple as whether or not the expected income net mortgage payments are greater than some percentage, such as an expected market return on the capital. The selling decision could be similarly motivated, or driven by households that age out. Investment buyers and sellers could participate in the existing sales market, with the addition of flagging an investment property as “for rent” rather than “owner occupied”, where it would be pickup up as a rental property in the rental market.
#### Evolution of Housing Stock
Houses age and fall into disrepair, or demand for a certain quality of housing may change over time, both of which would lead to some houses being destroyed and new houses built. Housing stock could also expand according to demand, but wouldn’t necessarily contract. With the observer acting as construction company, houses of a specific age or quality could be removed and replaced with new houses based on demand for specific quality, determined by the local price.
#### Competing Banks
The model currently has the option on the interface to select the number of banks, which means that mortgage holders are randomly assigned one of the banks as a lender, but their behaviour is now identical. Banks could make decisions about LTV ratios or interest rates for various borrowers and properties based on profit optimization, and households could chose the best mortgage available from a variety of offers.

@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

house efficiency
false
0
Rectangle -7500403 true true 180 90 195 195
Rectangle -7500403 true true 90 165 210 255
Rectangle -16777216 true false 165 195 195 255
Rectangle -16777216 true false 105 202 135 240
Polygon -7500403 true true 225 165 75 165 150 90
Line -16777216 false 75 165 225 165

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
